/* Copyright (c) 2005 MichalDominik@gmail.com ; MIT X11 license
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
 * SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
 * THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

using System;
using System.Runtime.InteropServices;
using Gtk;

public class Counter {
        
        delegate bool CounterDelegate (IntPtr val);
        static CounterDelegate counterUpdater = new CounterDelegate (OnCounterUpdated);
        // Note, that counterUpdater HAS to be static not only beacuse we
        // want to access it from Main. We don't want the GC to collect it. 
        
        [DllImport("counter")]
        static extern void SpawnCounterThread (CounterDelegate del);

        // Gtk stuff
        static Window myWin;
        static Label myLabel;
        
        
        // Main routine
        public static void Main ()
        {
                Application.Init ();

                // Create all the gtk stuff
                myWin = new Window ("Threaded Counter");
                myLabel = new Label ();
                myWin.DefaultWidth = 320;
                myWin.DefaultHeight = 240;
                myWin.Add (myLabel);
                myWin.ShowAll ();
                
                // Our unmanaged stuff
                Console.WriteLine ("MANAGED: Calling thread start");
                SpawnCounterThread (counterUpdater);
                Application.Run ();
        }

        
        // This is called by the g_main_loop, so we can safely
        // update the interface from here
        static bool OnCounterUpdated (IntPtr val)
        {
                Console.WriteLine ("MANAGED: Got counter update");
                Int32 time = (Int32) Marshal.ReadInt32 (val);
                myLabel.Text = String.Format ("Counter: {0}", time);
                return false;

                // Note, that the false returned here isn't returned to 
                // the calling thread itself. It's returned to the g_main_loop, 
                // to cancel further processing of this idle message.
        }

}
